#property copyright "Uniano"
#property link      "http://uniano.com"
#property version   "1.00"
#property strict
#include <Trade\PositionInfo.mqh>
#include <Trade\Trade.mqh>
#include <Trade\SymbolInfo.mqh> 

CTrade Trade;
CPositionInfo  m_position;                   // object of CPositionInfo class
CTrade         m_trade;                      // object of CTrade class
CSymbolInfo    m_symbol;  
//-----------------------------------------------------------------------------------------------------------------------------------------------
// External variables

input string                        Comment2             = "atrstops_v1 settings:";           // ................................................
input uint                          ATRPeriod            = 1;                                 // Period of ATR
input double                        Kv                   = 1.5;                               // Volatility by ATR
input string                        Comment3             = "Orders settings:";                // ................................................
input double                        Lot                  = 1;                              // Lots size
input ulong 								Magic						= 123;					                // Magic number
input double                        ProfitPercentage     = 1.0;                               // Profit Percentage
input double                        CloseLossPercent     = 1.0;                               // Zarar Yuzdesi (1.0 = 1%)
input group             "MACD"
input int                  Inp_MACD_fast_ema_period= 12;          // MACD: period for Fast average calculation
input int                  Inp_MACD_slow_ema_period= 26;          // MACD: period for Slow average calculation
input int                  Inp_MACD_signal_period  = 9;           // MACD: period for their difference averaging
input ENUM_APPLIED_PRICE   Inp_MACD_applied_price  = PRICE_CLOSE; // MACD: type of price
input double               InpDifference           = 0.00050;     // MACD Difference -> Close All
input group             "Additional features"
input bool     InpPrintLog          = false;       // Print log
input uchar    InpFreezeCoefficient = 1;           // Coefficient (if Freeze==0 Or StopsLevels==0)
input ulong    InpDeviation         = 10;          // Deviation, in Points (1.00045-1.00055=10 points)

//-----------------------------------------------------------------------------------------------------------------------------------------------
// Local variables
string signal = "",prefix = "Leledc";   
int handleatr = 0;
int      handle_iMACD; 
double buyatr = 0,sellatr = 0;
datetime indicatorscheck = 0;
double BuyOpenPrice = 0.0;
double SellOpenPrice = 0.0;
double static InitialBalance;
bool     m_need_close_all           = false;    // close all positions
bool     m_init_error               = false;  
//+------------------------------------------------------------------+
//| Tester function                                                  |
//+------------------------------------------------------------------+
double OnTester()
  {
    double AAA =0,BBB=0;
    if (TesterStatistics(STAT_EQUITY_DD) == 0) AAA = Point();
    else AAA = TesterStatistics(STAT_EQUITY_DD);
    if (TesterStatistics(STAT_PROFIT_FACTOR) >=200) BBB = 200; else BBB = TesterStatistics(STAT_PROFIT_FACTOR);
    if (TesterStatistics(STAT_TRADES)>=20) return((TesterStatistics(STAT_PROFIT)-AAA) /AAA * MathPow(BBB,0.333));
    else return(0);
  }
//+------------------------------------------------------------------+
// Initialization
int OnInit()
{
	// Tester check
	//if(MQLInfoInteger(MQL_TESTER))prefix="LeledcTest";
	
   // Magic number
   Trade.SetExpertMagicNumber(Magic);
	
	// Filling type
   if(IsFillingTypeAllowed(Symbol(),SYMBOL_FILLING_FOK))
     Trade.SetTypeFilling(ORDER_FILLING_FOK);
   else if(IsFillingTypeAllowed(Symbol(),SYMBOL_FILLING_IOC))
     Trade.SetTypeFilling(ORDER_FILLING_IOC);
   else Trade.SetTypeFilling(ORDER_FILLING_RETURN);
   
    m_need_close_all           = false;    // close all positions
   m_init_error               = false; 
     ResetLastError();
   if(!m_symbol.Name(Symbol())) // sets symbol name
     {
      Print(__FILE__," ",__FUNCTION__,", ERROR: CSymbolInfo.Name");
      return(INIT_FAILED);
     }
RefreshRates();
//---
   m_trade.SetExpertMagicNumber(Magic);
   m_trade.SetMarginMode();
   m_trade.SetTypeFillingBySymbol(m_symbol.Name());
   m_trade.SetDeviationInPoints(InpDeviation);
handle_iMACD=iMACD(m_symbol.Name(),Period(),Inp_MACD_fast_ema_period,Inp_MACD_slow_ema_period,
                      Inp_MACD_signal_period,Inp_MACD_applied_price);
   

	// ATR handle
	handleatr = iCustom(Symbol(),PERIOD_CURRENT,"ATRStopV2", ATRPeriod, Kv);
   if(handleatr < 0)
   {
     Print("Error with atrstops handle: ",GetLastError());
     return(-1);
   }

	return(INIT_SUCCEEDED);
}
//-----------------------------------------------------------------------------------------------------------------------------------------------
// Data of indicator  
double GetIndicatorValue(int handle,int buffer,int shift)
{
   double buf[];
   
   if(CopyBuffer(handle, buffer, shift, 1, buf)>0)
   return buf[0]; 
   else Comment(GetLastError());
   return EMPTY_VALUE;
}
//-----------------------------------------------------------------------------------------------------------------------------------------------
// Filling type
bool IsFillingTypeAllowed(string symbol,int fill_type) 
{ 
   int filling=(int)SymbolInfoInteger(symbol,SYMBOL_FILLING_MODE); 
   
   return((filling&fill_type)==fill_type); 
}
bool RefreshRates()
  {
//--- refresh rates
   if(!m_symbol.RefreshRates())
     {
      Print(__FILE__," ",__FUNCTION__,", ERROR: ","RefreshRates error");
      return(false);
     }
//--- protection against the return value of "zero"
   if(m_symbol.Ask()==0 || m_symbol.Bid()==0)
     {
      Print(__FILE__," ",__FUNCTION__,", ERROR: ","Ask == 0.0 OR Bid == 0.0");
      return(false);
     }
//---
   return(true);
  }
  
//-----------------------------------------------------------------------------------------------------------------------------------------------
// Processing
void OnTick()
{

	// Signal
	Signal();
	
	// Close orders
	CloseOrders();
	
	// Open orders
	OpenOrders();
}

void OnTradeTransaction(const MqlTradeTransaction &trans,
                        const MqlTradeRequest &request,
                        const MqlTradeResult &result)  
                        
                         {
//---
  }
void FreezeStopsLevels(double &freeze,double &stops)
  {

   double coeff=(double)InpFreezeCoefficient;
   if(!RefreshRates() || !m_symbol.Refresh())
      return;
//--- FreezeLevel -> for pending order and modification
   double freeze_level=m_symbol.FreezeLevel()*m_symbol.Point();
   if(freeze_level==0.0)
      if(InpFreezeCoefficient>0)
         freeze_level=(m_symbol.Ask()-m_symbol.Bid())*coeff;
//--- StopsLevel -> for TakeProfit and StopLoss
   double stop_level=m_symbol.StopsLevel()*m_symbol.Point();
   if(stop_level==0.0)
      if(InpFreezeCoefficient>0)
         stop_level=(m_symbol.Ask()-m_symbol.Bid())*coeff;
//---
   freeze=freeze_level;
   stops=stop_level;
//---
   return;
  }
//+------------------------------------------------------------------+
//| Get value of buffers                                             |
//+------------------------------------------------------------------+
bool iGetArray(const int handle,const int buffer,const int start_pos,
               const int count,double &arr_buffer[])
  {
   bool result=true;
   if(!ArrayIsDynamic(arr_buffer))
     {
      if(InpPrintLog)
         PrintFormat("ERROR! EA: %s, FUNCTION: %s, this a no dynamic array!",__FILE__,__FUNCTION__);
      return(false);
     }
   ArrayFree(arr_buffer);
//--- reset error code
   ResetLastError();
//--- fill a part of the iBands array with values from the indicator buffer
   int copied=CopyBuffer(handle,buffer,start_pos,count,arr_buffer);
   if(copied!=count)
     {
      //--- if the copying fails, tell the error code
      if(InpPrintLog)
         PrintFormat("ERROR! EA: %s, FUNCTION: %s, amount to copy: %d, copied: %d, error code %d",
                     __FILE__,__FUNCTION__,count,copied,GetLastError());
      //--- quit with zero result - it means that the indicator is considered as not calculated
      return(false);
     }
   return(result);
  }
//+------------------------------------------------------------------+
//| Search trading signals                                           |
//+------------------------------------------------------------------+
bool SearchTradingSignals(void)
  {
   double macd[],msignal[];
   ArraySetAsSeries(macd,true);
   ArraySetAsSeries(msignal,true);
   int start_pos=0,count=3;
   if(!iGetArray(handle_iMACD,MAIN_LINE,start_pos,count,macd) || !iGetArray(handle_iMACD,SIGNAL_LINE,start_pos,count,msignal))
      return(false);
//---
   if(macd[0]>signal[0])
      if(MathAbs(macd[0]-signal[0])>InpDifference)
         m_need_close_all=true;
         
         
         if(macd[0]<signal[0])
      if(MathAbs(signal[0]-macd[0])>InpDifference)
         m_need_close_all=true;
         
//---
   return(true);
  }
//+------------------------------------------------------------------+
//| Is position exists                                               |
//+------------------------------------------------------------------+
bool IsPositionExists(void)
  {
   for(int i=PositionsTotal()-1; i>=0; i--)
      if(m_position.SelectByIndex(i)) // selects the position by index for further access to its properties
         if(m_position.Symbol()==m_symbol.Name())
            return(true);
//---
   return(false);
  }
//+------------------------------------------------------------------+
//| Close all positions                                              |
//+------------------------------------------------------------------+
void CloseAllPositions(void)
  {
   double freeze=0.0,stops=0.0;
   FreezeStopsLevels(freeze,stops);
   
   for(int i=PositionsTotal()-1; i>=0; i--) // returns the number of current positions
      if(m_position.SelectByIndex(i)) // selects the position by index for further access to its properties
         if(m_position.Symbol()==m_symbol.Name())
           {
            if(m_position.PositionType()==POSITION_TYPE_BUY)
              {
               bool take_profit_level=(m_position.TakeProfit()!=0.0 && m_position.TakeProfit()-m_position.PriceCurrent()>=freeze) || m_position.TakeProfit()==0.0;
               bool stop_loss_level=(m_position.StopLoss()!=0.0 && m_position.PriceCurrent()-m_position.StopLoss()>=freeze) || m_position.StopLoss()==0.0;
               if(take_profit_level && stop_loss_level)
                  if(!m_trade.PositionClose(m_position.Ticket())) // close a position by the specified m_symbol
                     if(InpPrintLog)
                        Print(__FILE__," ",__FUNCTION__,", ERROR: ","BUY PositionClose ",m_position.Ticket(),", ",m_trade.ResultRetcodeDescription());
              }
            if(m_position.PositionType()==POSITION_TYPE_SELL)
              {
               bool take_profit_level=(m_position.TakeProfit()!=0.0 && m_position.PriceCurrent()-m_position.TakeProfit()>=freeze) || m_position.TakeProfit()==0.0;
               bool stop_loss_level=(m_position.StopLoss()!=0.0 && m_position.StopLoss()-m_position.PriceCurrent()>=freeze) || m_position.StopLoss()==0.0;
               if(take_profit_level && stop_loss_level)
                  if(!m_trade.PositionClose(m_position.Ticket())) // close a position by the specified m_symbol
                     if(InpPrintLog)
                        Print(__FILE__," ",__FUNCTION__,", ERROR: ","SELL PositionClose ",m_position.Ticket(),", ",m_trade.ResultRetcodeDescription());
              }
           }
  }
//-----------------------------------------------------------------------------------------------------------------------------------------------
// Signal
void Signal()
{
   signal = "";
   
   if(indicatorscheck!=iTime(Symbol(),PERIOD_CURRENT,0))
   {
      buyatr          = GetIndicatorValue(handleatr,2,1);
      sellatr         = GetIndicatorValue(handleatr,3,1);
      indicatorscheck = iTime(Symbol(),PERIOD_CURRENT,0);
     
     
   }
   //Comment(buyatr,"   ",sellatr);
   if(buyatr!=EMPTY_VALUE && buyatr!=0)signal = "buy";
   if(sellatr!=EMPTY_VALUE && sellatr!=0)signal = "sell";
   
   
}
//-----------------------------------------------------------------------------------------------------------------------------------------------
// Close orders
void CloseOrders()
{
   double ASK = SymbolInfoDouble(Symbol(), SYMBOL_ASK);
   double BID = SymbolInfoDouble(Symbol(), SYMBOL_BID);
   double ATRBuy = GetIndicatorValue(handleatr,0,1);
   double ATRSell = GetIndicatorValue(handleatr,1,1);
  
   int Bizad = CountOpenedPositions("all");
   
   if(CountOpenedPositions("buy")!=0 && 
     (signal == "sell" || 
     ((BID - ATRBuy)/ATRBuy > ProfitPercentage/100.0))){CloseAllOrders("buy"); Comment("Closed By ATR");}
     
   else if(CountOpenedPositions("sell")!=0 && 
     (signal == "buy" ||  
     ((ATRSell - ASK)/ATRSell > ProfitPercentage/100.0))) {CloseAllOrders("sell"); Comment("Closed By ATR");}
   
   else if
    ((CountOpenedPositions("buy") > 0 &&
             
            ((ASK + BID)/InitialBalance / 2.0  <= (100 - CloseLossPercent)/100.0)) ||
            
            (CountOpenedPositions("sell") > 0 &&
            
            ((ASK + BID)/InitialBalance / 2.0  >= (100 + CloseLossPercent)/100.0)))
            
            
            
           
          
            
      {
      CloseAllOrders("buy");
      CloseAllOrders("sell");
      if (InitialBalance != 0.0) Comment("Closed by Initial deposit");
      InitialBalance = 0.0;
      }
     
  //Comment(DoubleToString(InitialBalance * 2.0/ (ASK + BID)),"  ",(100+CloseProfitPercent)/100.0,"  ",(100-CloseLossPercent)/100.0);   
}
//-----------------------------------------------------------------------------------------------------------------------------------------------
// Open orders
void OpenOrders()
{
   datetime currentopened = (GlobalVariableCheck(prefix+(string)Magic)?(datetime)GlobalVariableGet(prefix+(string)Magic):0);
	datetime currentbar	  = iTime(Symbol(),PERIOD_CURRENT,0);
   double ASK = SymbolInfoDouble(Symbol(), SYMBOL_ASK);
   double BID = SymbolInfoDouble(Symbol(), SYMBOL_BID);
   double ATRBuy = GetIndicatorValue(handleatr,0,1);
   double ATRSell = GetIndicatorValue(handleatr,1,1);
   
// Open orders
if(currentopened!=currentbar)
{
// Buy
   if(signal=="buy")
   {
      if(!Trade.Buy(Lot,NULL,0,0,0,DoubleToString(ASK,5)))
   		{Print("Trade.Buy error #",GetLastError());
          InitialBalance = ASK;
          Comment("");}
          
      else GlobalVariableSet(prefix+(string)Magic,(double)currentbar);
   }
   
// Sell
   if(signal=="sell")
   {
      if(!Trade.Sell(Lot,NULL,0,0,0,DoubleToString(BID,5)))
   		{Print("Trade.Sell error #",GetLastError()); 
          InitialBalance = BID;
          Comment("");}
          
   	else GlobalVariableSet(prefix+(string)Magic,(double)currentbar);
   }
}
}

// Count opened positions
int CountOpenedPositions(string type)
{
int count = 0;
   
for (int i = PositionsTotal() - 1; i>=0; i--)
{
   if(PositionSelectByTicket(PositionGetTicket(i)))
   {
      if(PositionGetInteger(POSITION_MAGIC) == Magic)
      {
         if(type=="all")
           count ++; 
         if(type=="sell" && PositionGetInteger(POSITION_TYPE)==POSITION_TYPE_SELL)
           count ++; 
         if(type=="buy" && PositionGetInteger(POSITION_TYPE)==POSITION_TYPE_BUY)
           count ++; 
         if(PositionGetInteger(POSITION_TYPE)==POSITION_TYPE_SELL) SellOpenPrice = PositionGetDouble(POSITION_PRICE_OPEN);
         if(PositionGetInteger(POSITION_TYPE)==POSITION_TYPE_BUY) BuyOpenPrice = PositionGetDouble(POSITION_PRICE_OPEN);
         
         InitialBalance = StringToDouble(PositionGetString(POSITION_COMMENT));
      } 
   }	
}

return count;
}
//-----------------------------------------------------------------------------------------------------------------------------------------------
// Close all orders
void CloseAllOrders(string type)
{
for (int s = PositionsTotal() - 1; s>=0; s--)
{
   if(PositionSelectByTicket(PositionGetTicket(s)))
   { 
      if(PositionGetInteger(POSITION_MAGIC) == Magic)
      {
         ulong ticket  = PositionGetTicket(s);
         int typeorder = (int)PositionGetInteger(POSITION_TYPE);
         
         if(type=="all")
         {
            if(!Trade.PositionClose(ticket))
   		      Print("Trade.PositionClose has been ended with error #",GetLastError());
		   }
		   if(type=="buy" && typeorder==0)
         {
            if(!Trade.PositionClose(ticket))
   		      Print("Trade.PositionClose has been ended with error #",GetLastError());
		   }
		   if(type=="sell" && typeorder==1)
         {
            if(!Trade.PositionClose(ticket))
   		      Print("Trade.PositionClose has been ended with error #",GetLastError());
		   }
	   }
   }
}
}
//-----------------------------------------------------------------------------------------------------------------------------------------------